/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.gameserver;

import org.ugasp.eventmodel.event.DataEvent;
import org.ugasp.core.ApplicationInstance;

/**
 * A game server logic running on GASP must implement this class GASPServer.
 * 
 * @author PELLERIN Romain (pellerin@cnam.fr) - MEGA Project 
 */
public interface Server {
	
    /**
     * Link the application instance to the game server logic.
     * 
     * @param applicationInstance
     */
    public void setOwnerAI(ApplicationInstance applicationInstance);
    
    /**
     * Send to the linked applicationInstance a DataEvent for a specific actorSession.
     * 
     * @param actorSessionID
     * @param dataEvent the DataEvent to send
     */
    public void sendDataTo(int actorSessionID, DataEvent dataEvent);
    
    /**
     * Retrieve the CustomTypes encoder/decoder.
     */
	public org.ugasp.protocol.moods.CustomTypes getCustomTypes();
	
	/**
     * Retrieve the CustomTypes string based encoder/decoder.
     */
	public org.ugasp.protocol.moods.string.CustomTypes getCustomTypes4StringEncoding();
    
    /**
     * Retrieve the actor ID linked to the actor session ID.
     * (persistency) Provide the storage of player game informations. 
     * 
     * @param actorSessionID
     * @param dataEvent the DataEvent to send
     */
    public int retrieveActorID(int actorSessionID);
    
    
    /**
     * Retrieve the application ID.
     * (persistency) Provide the storage of player game informations. 
     */
    public int retrieveApplicationID();
    
    /**
     * Send to the linked applicationInstance a DataEvent for a specific actorSession.
     * 
     * @param actorSessionID
     * @param dataEvent the DataEvent to send
     */
    /** TODO: � virer 
     	protected void forceQuitEvent(int actorSessionID){
        if (appIns.getActorSession(actorSessionID)!=null) 
            appIns.onQuitEvent(new QuitEventImpl(actorSessionID));
    }*/
    
/*    
    *//**
     * Method called when a JoinEvent is received from the platform.
     * 
     * @param je a JoinEvent
     *//*
    public void onJoinEvent(JoinEvent je);

    
    *//**
     * Method called when a StartEvent is received from the platform.
     * 
     * @param se a StartEvent
     *//*
    public void onStartEvent(StartEvent se);

    
    *//**
     * Method called when a EndEvent is received from the platform.
     * 
     * @param ee a EndEvent
     *//*
    public void onEndEvent(EndEvent ee);

    
    *//**
     * Method called when a QuitEvent is received from the platform.
     * 
     * @param qe a QuitEvent
     *//*
    public void onQuitEvent(QuitEvent qe);

    
    *//**
     * Method called when a DataEvent is received from the platform.
     * 
     * @param de a DataEvent
     *//*
    public void onDataEvent(DataEvent de);*/

}
