/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.impl.connector.http;

//#include parameters.h
////#define LOGGING
////#define DEBUG_CONSOLE
////#define DEBUG_ALERTSCREEN
//#endinclude

//#ifdef LOGGING
//# import org.apache.log4j.Category;
//#endif LOGGING

import org.osgi.framework.BundleContext;
import org.osgi.service.http.HttpContext;
import org.osgi.service.http.HttpService;
import org.osgi.service.http.NamespaceException;

import org.ugasp.core.ActorSession;
import org.ugasp.core.ApplicationInstance;
import org.ugasp.core.MasterApplicationInstance;
import org.ugasp.core.Session;
import org.ugasp.service.external.admin.AdminConnector;
import org.ugasp.service.external.player.PlayerConnector;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Iterator;


import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * This servlet is call the first time the user log in with the Application.
 * The HTTP parameters are :
 * - uid the UserID of the operator
 * - appID the ApplicationID hard coded in the application.
 * - username the choosed username
 * - password the choosed password
 * 
 * @author PELLERIN Romain (pellerin@cnam.fr) - MEGA Project 
 */
public class BinaryServlet extends HttpServlet implements org.ugasp.connector.http.Servlet/*, Runnable*/{
    //#ifdef LOGGING
    //# private Category cat;
    //#endif LOGGING
    private HttpService httpService;
    private PlayerConnector playerConnector;
    
    private static final String SERVLET_URL = "/uGASP/BinaryServlet";
    
    public BinaryServlet(){
    	System.out.println("BinaryServlet instantiated");
    }
    
    public void init(){ System.out.println("BinaryServlet init");}
    
    public void start(){
    	System.out.println("BinaryServlet started !");
    }
    
    public void stop(){
    	System.out.println("BinaryServlet stopped !");
    }
    
    public void bindPlayerConnector(PlayerConnector connector){
    	playerConnector = connector;
    	System.out.println("PlayerConnector "+SERVLET_URL+" binded: "+connector);
    }
    
    public void unbindPlayerConnector(){
    	playerConnector = null;
    	System.out.println("unbindPlayerConnector");
    }
    
    
    public void bindHttpService(HttpService service){
    	try {
    		// get the httpService from bundle main servlet
    		httpService = service;
    		
	    	// create a default context to share between registrations
	        HttpContext httpContext = httpService.createDefaultHttpContext();

	        // register the hello world servlet
	        Dictionary initParams = new Hashtable();
	        initParams.put( "from", "HttpService" );

	        httpService.registerServlet(SERVLET_URL, this, initParams, httpContext);
		    System.out.println("Servlet "+SERVLET_URL+" registered to HTTP service");
    	} catch (Exception e) {
    		System.out.println("Exception in bindHttpService method:"+e.getMessage());
    		e.printStackTrace();
    	}
    }
    
    public void unbindHttpService(){
    	httpService.unregister(SERVLET_URL);
    	System.out.println("Servlet "+SERVLET_URL+" unregistered to HTTP service");
    	//httpService=null;
    }
    
    public void doGet(HttpServletRequest request,HttpServletResponse response) throws IOException, ServletException {}
    
  

	public void doPost(HttpServletRequest request,HttpServletResponse response) throws IOException, ServletException {
    	DataInputStream dis = null;
        try{
        	dis = new DataInputStream(request.getInputStream());
        	byte requesterType = dis.readByte();
        	System.out.println("BinaryServlet requesterType: "+requesterType);
	    	switch(requesterType){
	    		case 0: // Admin request
	    			doPostAdmin(dis,response);
	    			break;
	    			
	    		case 1: // GameMaster
	    			break;
	    			
	     		case 2: // Player request
	     			doPostPlayer(dis,response);
	     			break;
	     			
	     		case 3: // Sensor request
	     			break;
	     		default:
	     			System.out.println("Bad client type: "+requesterType);
	     			break;
	    	}
        }
        catch(Exception e){
        	System.out.println("Exception occured during in game binary communication: "+e.getMessage());
		}
	    finally{
	        if (dis!=null) try{dis.close();} catch (Exception e){}
	    }
    }
	
	private void doPostAdmin(DataInputStream dis, HttpServletResponse response) {
		// for future distant management
	}	

	
	public void doGetPlayer(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
		// TODO: in game for flash client?
	}
	
	
	public void doPostPlayer(DataInputStream dis, HttpServletResponse response) throws IOException, ServletException {
	    DataOutputStream dos = null;
	    try{
			short aSID = dis.readShort();

			playerConnector.sendData(aSID, dis);
			dis.close();

			dos = new DataOutputStream(response.getOutputStream());
			
			// #ifdef LOGGING
			// # cat.info("---------------- START ENCODING ON HTTP
			// REQUEST DOS ----------------");
			// #endif LOGGING

			// send events to client
			playerConnector.getEvents(aSID, dos);
			
			// #ifdef LOGGING
			// # cat.info("---------------- END of ENCODING ON HTTP
			// REQUEST DOS ----------------");
			// # cat.debug("InGameServlet: encoding events to client ok,
			// nb events="+i);
			// #endif LOGGING
		}
		catch(Exception e){
			System.out.println("Exception occured during in game binary communication, may be input stream is invalid: "+e.getMessage());
		}
	    finally{
	        if (dos!=null) try{dos.close();} catch (Exception e){}
	        if (dis!=null) try{dis.close();} catch (Exception e){}
	    }
	}
}
