/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.impl.connector.http;

//#include parameters.h
////#define LOGGING
////#define DEBUG_CONSOLE
////#define DEBUG_ALERTSCREEN
//#endinclude

//#ifdef LOGGING
//# import org.apache.log4j.Category;
//#endif LOGGING

import org.osgi.framework.BundleContext;
import org.osgi.service.http.HttpContext;
import org.osgi.service.http.HttpService;
import org.osgi.service.http.NamespaceException;

import org.ugasp.core.ActorSession;
import org.ugasp.core.ApplicationInstance;
import org.ugasp.core.MasterApplicationInstance;
import org.ugasp.core.Session;
import org.ugasp.service.external.admin.AdminConnector;
import org.ugasp.service.external.player.PlayerConnector;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Iterator;


import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * This servlet is call the first time the user log in with the Application.
 * The HTTP parameters are :
 * - uid the UserID of the operator
 * - appID the ApplicationID hard coded in the application.
 * - username the choosed username
 * - password the choosed password
 * 
 * @author PELLERIN Romain (pellerin@cnam.fr) - MEGA Project 
 */
public class Servlet extends HttpServlet implements org.ugasp.connector.http.Servlet/*, Runnable*/{
	
    //iPojo dynamic binding fields
    private PlayerConnector playerConnector;
    private AdminConnector adminConnector;
    private HttpService httpService;

    //other fields
    private BinaryServlet binaryServlet;
    
    //#ifdef LOGGING
    //# private Category cat;
    //#endif LOGGING
    
    private BundleContext bundleContext;
    
    private static final String SERVLET_URL = "/uGASP/Servlet";
    private Thread thread;
    
    public Servlet(BundleContext bc){
    	bundleContext = bc;
	    // instanciate BinaryServlet
	    binaryServlet = new BinaryServlet();
	    
    	System.out.println("Servlet instantiated");
    }
    
    public void start(){
    	binaryServlet.start();
    	System.out.println("Servlet started !");
    }
    
    public void stop(){
    	binaryServlet.stop();
    	System.out.println("Servlet stopped !");
    }
    
    public void bindPlayerConnector(PlayerConnector connector){
    	binaryServlet.bindPlayerConnector(connector);
    	System.out.println("PlayerConnector "+SERVLET_URL+" binded: "+connector);
    }
    
    public void unbindPlayerConnector(){
    	binaryServlet.unbindPlayerConnector();
    	System.out.println("unbindPlayerConnector");
    }
    
    protected PlayerConnector getPlayerConnector(){
    	return playerConnector;
    }
    
    
    public void bindHttpService(HttpService service){
    	try {    		
	    	// create a default context to share between registrations
	        HttpContext httpContext = httpService.createDefaultHttpContext();

	        // register the hello world servlet
	        Dictionary initParams = new Hashtable();
	        initParams.put( "from", "HttpService" );

	        httpService.registerServlet(SERVLET_URL, this, initParams, httpContext);	        
		    System.out.println("Servlet "+SERVLET_URL+" registered to HTTP service");
		    
		    // instanciate BinaryServlet
		    binaryServlet.bindHttpService(httpService);
		    
    	} catch (Exception e) {
    		System.out.println("Exception in bindHttpService method:"+e.getMessage());
    		e.printStackTrace();
    	}
    }
    
    public void unbindHttpService(){
    	binaryServlet.unbindHttpService();
    	httpService.unregister(SERVLET_URL);
    	System.out.println("Servlet "+SERVLET_URL+" unregistered to HTTP service");
    	//httpService=null;
    }
    
    public void doGet(HttpServletRequest request,HttpServletResponse response) throws IOException, ServletException {
    	int requesterType = Integer.parseInt(request.getParameter("from"));
    	//int language =  Integer.parseInt(request.getParameter("type"));
    	
    	switch(requesterType){
    		case 0: // Admin request
    			doGetAdmin(request,response);
    			break;
    			
    		case 1: // GameMaster
    			break;
    			
     		case 2: // Player request
     			doGetPlayer(request,response);
     			break;
     			
     		case 3: // Sensor request
     			break;
    	}
    }
    
  

	public void doPost(HttpServletRequest request,HttpServletResponse response) throws IOException, ServletException {
    	int requesterType = Integer.parseInt(request.getParameter("from"));
    	//int language =  Integer.parseInt(request.getParameter("type"));
    	
    	switch(requesterType){
    		case 0: // Admin request
    			doPostAdmin(request,response);
    			break;
    			
    		case 1: // GameMaster
    			break;
    			
     		case 2: // Player request
     			doPostPlayer(request,response);
     			break;
     			
     		case 3: // Sensor request
     			break;
    	}
    }
	

	private void doGetAdmin(HttpServletRequest request,HttpServletResponse response) throws IOException {
		String mode = request.getParameter("mode");
		
		// admin features
		if (mode.equals("platformState")){
			AdminWebPages.renderPlatformState(request,response,adminConnector);
		}
		
	}
	

	private void doPostAdmin(HttpServletRequest request,HttpServletResponse response) {
		// for future distant management
	}	

	
	public void doGetPlayer(HttpServletRequest request,HttpServletResponse response) throws IOException, ServletException {
		PrintWriter out = null;
		try{
		    System.out.println("doGet called");
		    System.out.println("httpService:"+ httpService);
		    System.out.println("PlayerConnector:"+ playerConnector);

	    	// get http request parameters for login 
		    String mode = request.getParameter("mode");

			// FirstLogin features
			if (mode.equals("FirstLogin")){
				try{
					short uid = Short.parseShort(request.getParameter("uid"));
					short appID = Short.parseShort(((String) request.getParameter("appID")));
					String username = request.getParameter("username");
					String password = request.getParameter("password");
					
					int aID = playerConnector.firstLogin(uid,appID,username,password);
					
					response.setContentType("application/x-www-urlform-encoded; charset=UTF-8");
					out = new PrintWriter(response.getOutputStream());
					out.println("aID="+aID);
				}
				catch(Exception e){
					System.out.println("Exception occured during FirstLogin, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Login features
			else if (mode.equals("Login")){
				try{
					// get http request parameters for login 
					short aID = Short.parseShort(request.getParameter("aID"));
					String username = request.getParameter("username");
					String password = request.getParameter("password");
					String version = request.getParameter("version");

					int sID = playerConnector.login(aID, username, password);

					response.setContentType("application/x-www-urlform-encoded; charset=UTF-8");
					out = new PrintWriter(response.getOutputStream());
					out.println("sID=" + sID);
				}
				catch(Exception e){
					System.out.println("Exception occured during Login, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Lobby features
			else if (mode.equals("Lobby")){
				try{
					String sID = request.getParameter("sID");
			    	String data = playerConnector.lobby(Short.parseShort(sID));
					
					// send the lobby encoded string to Flash client
					response.setContentType("application/x-www-urlform-encoded; charset=UTF-8");
					out = new PrintWriter(response.getOutputStream());
					out.println("data=" + data);
				}
				catch(Exception e){
					System.out.println("Exception occured during Lobby, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Name features
			else if (mode.equals("Name")){
				try{
			    	// get http request parameters for login
					short aSID = Short.parseShort(request.getParameter("aSID"));
					String name = request.getParameter("name");

					String pseudo = playerConnector.name(aSID,name);
					
					//#ifdef LOGGING
					//# cat.debug("Name: aSID n�" + aSID + " renamed to " + aS.getPseudoName());
					//#endif LOGGING
					
					// write the short sID in the http response
					response.setContentType("application/x-www-urlform-encoded; charset=UTF-8");
					out = new PrintWriter(response.getOutputStream());
					out.println("pseudoname=" + pseudo);

					//#ifdef LOGGING
					//# cat.debug("username sended to Flash: " + aS.getPseudoName());
					//#endif LOGGING
					
				}
				catch(Exception e){
					System.out.println("Exception occured during Name, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Join features
			else if (mode.equals("Join")){
				try{
					//get http request parameters for login 
				    short sID =  Short.parseShort(request.getParameter("sID"));
					String applicationInstanceID = request.getParameter("aIID");
					
					short aIID;
					short aSID;
					
					if (applicationInstanceID.equals("")) aIID = 0;
					else aIID = Short.parseShort(applicationInstanceID);
					
					aSID = playerConnector.joinAI(sID,aIID);
					
					// write the short sID in the http response
					response.setContentType("application/x-www-urlform-encoded; charset=UTF-8");
					out = new PrintWriter(response.getOutputStream());
					out.println("aSID=" + aSID);
				}
				catch(Exception e){
					System.out.println("Exception occured during Join, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Create features
			else if (mode.equals("Create")){
				try{
					// get http request parameters for login
					short sID = Short.parseShort(request.getParameter("sID"));
					int minActors = Integer.parseInt(request.getParameter("minActors"));
					int maxActors = Integer.parseInt(request.getParameter("maxActors"));
					String[] actors = request.getParameterValues("actors");
					
					int aSID = playerConnector.create(sID, minActors, maxActors, actors); 

					// write the short sID in the http response
					response.setContentType("application/x-www-urlform-encoded; charset=UTF-8");
					out = new PrintWriter(response.getOutputStream());
					out.println("aSID=" + aSID);
				}
				catch(Exception e){
					System.out.println("Exception occured during Create, may be parameters are invalid: "+e.getMessage());
				}
			}
			// StartAI features
			else if (mode.equals("StartAI")){
				try{
					//get http request parameters for login 
					short aSID =  Short.parseShort(request.getParameter("aSID"));
					
					playerConnector.start(aSID);
				}
				catch(Exception e){
					System.out.println("Exception occured during StartAI, may be parameters are invalid: "+e.getMessage());
				}
			}
			// InGame features
			else if (mode.equals("InGame")){
				try{
					short aSID = Short.parseShort(request.getParameter("aSID"));
					String data = request.getParameter("data");
					
					playerConnector.sendData(aSID,data);
					
					//#ifdef LOGGING
					//# cat.info("---------------- START ENCODING ON HTTP REQUEST DOS ----------------");
					//#endif LOGGING
					
					// send events to client
					data = playerConnector.getEvents(aSID);
					
					//#ifdef LOGGING
					//# cat.info("---------------- END of ENCODING ON HTTP REQUEST DOS ----------------");
					//# cat.debug("InGameServlet: encoding events to flash client ok");
					//#endif LOGGING
					
					// write the short sID in the http response
					response.setContentType("application/x-www-urlform-encoded; charset=UTF-8");
					out = new PrintWriter(response.getOutputStream());
					out.println("data=" + data);
				}
				catch(Exception e){
					System.out.println("Exception occured during InGame, may be parameters are invalid: "+e.getMessage());
				}
			}
			// EndAI features
			else if (mode.equals("EndAI")){
				try{
					// get http request parameters for login 
					short aSID =  Short.parseShort(request.getParameter("aSID"));
					
					playerConnector.end(aSID);
				}
				catch(Exception e){
					System.out.println("Exception occured during EndAI, may be parameters are invalid: "+e.getMessage());
				}
			}
			// QuitAI features
			else if (mode.equals("QuitAI")){
				try{
					// get http request parameters for login 
					short aSID =  Short.parseShort(request.getParameter("aSID"));
					
					playerConnector.quitAI(aSID);
				}
				catch(Exception e){
					System.out.println("Exception occured during QuitAI, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Quit features
			else if (mode.equals("Quit")){
				try{
					// get http request parameters for login 
					short aSID =  Short.parseShort(request.getParameter("aSID"));
					
					playerConnector.quit(aSID);
				}
				catch(Exception e){
					System.out.println("Exception occured during Quit, may be parameters are invalid: "+e.getMessage());
				}
			}
			
			
	    }
	    catch (Exception e){
	    	//#ifdef LOGGING
	        //# cat.info("cannot complied to received first login request");
	        //#endif LOGGING
	    }
	    finally{if (out!=null) try{out.close();} catch (Exception e){}}
	}
	
	
	public void doPostPlayer(HttpServletRequest request,HttpServletResponse response) throws IOException, ServletException {
	    DataOutputStream dos = null;
	    DataInputStream dis = null;
	    	
	    try{
	    	// get http request parameters for login 
			String mode = request.getParameter("mode");
	    	
	    	// FirstLogin features
			if (mode.equals("FirstLogin")){
				try{
			    	// get http request parameters for login 
					short uid = Short.parseShort(request.getParameter("uid"));
					short appID = Short.parseShort(((String) request.getParameter("appID")));
					String username = request.getParameter("username");
					String password = request.getParameter("password");
					
					// registration of the user with his choice of username and password
					// return a short aID, permanent actorID associated with the userID and applicationID
					// or the short "0" if the user have not the appropriate rights
					int aID = playerConnector.firstLogin(uid,appID,username,password);
					
					// write the short aID in the http response
					dos = new DataOutputStream(response.getOutputStream());
				
					dos.writeShort(Short.parseShort(aID+""));
				}
				catch(Exception e){
					System.out.println("Exception occured during FirstLogin, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Login features
			else if (mode.equals("Login")){
				try{
					short aID =  Short.parseShort(request.getParameter("aID"));
					String username = request.getParameter("username");
					String password = request.getParameter("password");
					String version = request.getParameter("version");
					
					int sID = playerConnector.login(aID, username, password);
					
					System.out.println("Login: sID returned to user: "+sID);
					
					// write the short sID in the http response
					dos = new DataOutputStream(response.getOutputStream());
					dos.writeShort(Short.parseShort(sID+""));
				}
				catch(Exception e){
					System.out.println("Exception occured during Login, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Lobby features
			else if (mode.equals("Lobby")){
				try{
					String sID = request.getParameter("sID");
					dos = new DataOutputStream(response.getOutputStream());
					
					playerConnector.lobby(Short.parseShort(sID),dos);
				}
				catch(Exception e){
					System.out.println("Exception occured during Lobby, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Name features
			else if (mode.equals("Name")){
				try{
					 //get http request parameters for login 
				    short aSID =  Short.parseShort(request.getParameter("aSID"));
					String name = request.getParameter("name");
			
					String pseudo = playerConnector.name(aSID,name);

					//#ifdef LOGGING
					//# cat.debug("Name: aSID n�"+aSID+ " renamed to "+aS.getPseudoName());
					//#endif LOGGING
					
					//send to the player his new pseudoname correctly formed
					dos = new DataOutputStream(response.getOutputStream());
				    dos.writeUTF(pseudo);
				}
				catch(Exception e){
					System.out.println("Exception occured during Name, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Join features
			else if (mode.equals("Join")){
				try{
					 //get http request parameters for login 
				    short sID =  Short.parseShort(request.getParameter("sID"));
					String applicationInstanceID = request.getParameter("aIID");
					
					//#ifdef LOGGING
					//# cat.debug("JoinServlet: called");
					//#endif LOGGING
					
					short aIID;
					short aSID;
					
					if (applicationInstanceID.equals("")) aIID = 0;
					else aIID = Short.parseShort(applicationInstanceID);
					
					aSID = playerConnector.joinAI(sID,aIID);
					
					// write the short sID in the http response
					dos = new DataOutputStream(response.getOutputStream());
					dos.writeShort(aSID);
				}
				catch(Exception e){
					System.out.println("Exception occured during Lobby, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Create features
			else if (mode.equals("Create")){
				try{
					// get http request parameters for login 
					short sID =  Short.parseShort(request.getParameter("sID"));
					int minActors = Integer.parseInt(request.getParameter("minActors"));
					int maxActors = Integer.parseInt(request.getParameter("maxActors"));
					String[] actors = request.getParameterValues("actors");
					
					int aSID = playerConnector.create(sID, minActors, maxActors, actors); 
					
					dos = new DataOutputStream(response.getOutputStream());
					dos.writeShort(aSID);	
				}
				catch(Exception e){
					System.out.println("Exception occured during Create, may be parameters are invalid: "+e.getMessage());
				}
			}
			// StartAI features
			else if (mode.equals("StartAI")){
				try{
					//get http request parameters for login 
					short aSID =  Short.parseShort(request.getParameter("aSID"));
					
					boolean b = playerConnector.start(aSID);
					
					dos = new DataOutputStream(response.getOutputStream());
					dos.writeBoolean(b);
				}
				catch(Exception e){
					System.out.println("Exception occured during StartAI, may be parameters are invalid: "+e.getMessage());
				}
			}
			// InGame features
			else if (mode.equals("InGame")){
				try{
					dis = new DataInputStream(request.getInputStream());
					short aSID = dis.readShort();

					playerConnector.sendData(aSID, dis);
					dis.close();

					dos = new DataOutputStream(response.getOutputStream());

					// #ifdef LOGGING
					// # cat.info("---------------- START ENCODING ON HTTP
					// REQUEST DOS ----------------");
					// #endif LOGGING

					// send events to client
					playerConnector.getEvents(aSID, dos);

					// #ifdef LOGGING
					// # cat.info("---------------- END of ENCODING ON HTTP
					// REQUEST DOS ----------------");
					// # cat.debug("InGameServlet: encoding events to client ok,
					// nb events="+i);
					// #endif LOGGING
				}
				catch(Exception e){
					System.out.println("Exception occured during InGame, may be parameters are invalid: "+e.getMessage());
				}
			}
			// EndAI features
			else if (mode.equals("EndAI")){
				try{
					// get http request parameters for login 
					short aSID =  Short.parseShort(request.getParameter("aSID"));
					
					playerConnector.end(aSID);
				}
				catch(Exception e){
					System.out.println("Exception occured during EndAI, may be parameters are invalid: "+e.getMessage());
				}
			}
			// QuitAI features
			else if (mode.equals("QuitAI")){
				try{
					// get http request parameters for login 
					short aSID =  Short.parseShort(request.getParameter("aSID"));
					
					playerConnector.quitAI(aSID);
				}
				catch(Exception e){
					System.out.println("Exception occured during QuitAI, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Quit features
			else if (mode.equals("Quit")){
				try{
					// get http request parameters for login 
					short sID =  Short.parseShort(request.getParameter("sID"));
					
					playerConnector.quit(sID);
				}
				catch(Exception e){
					System.out.println("Exception occured during Quit, may be parameters are invalid: "+e.getMessage());
				}
			}
						
	    }
	    catch (Exception e){
	    	//#ifdef LOGGING
	        //# cat.info("cannot complied to received first login request");
	        //#endif LOGGING
	    }
	    finally{
	        if (dos!=null) try{dos.close();} catch (Exception e){}
	        if (dis!=null) try{dis.close();} catch (Exception e){}
	    }
	}
}
