package org.ugasp.impl.connector.socket;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;

import org.ugasp.service.external.admin.AdminConnector;
import org.ugasp.service.external.player.PlayerConnector;

public class ClientManager implements Runnable{
	
	private PlayerConnector playerConnector;
	private AdminConnector adminConnector;
	
	private Thread thread;
	private DataInputStream dis;
	private DataOutputStream dos;
	protected Socket socket;
	private String hostname;
	
	public ClientManager(AdminConnector admin, PlayerConnector player, Socket s){
		playerConnector = player;
		adminConnector = admin;
		socket = s;
		hostname = socket.getInetAddress().getHostAddress();
		
		try{
			//in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
			dis = new DataInputStream(socket.getInputStream());
			dos = new DataOutputStream(socket.getOutputStream());
			
			System.out.println("Buffer size input: "+socket.getReceiveBufferSize());
			System.out.println("Buffer size output: "+socket.getSendBufferSize());
			thread = new Thread(this);
			thread.start();
		} 
		catch (IOException e) {
			System.out.println("Open i/o streams on socket failed: "+e.getMessage());
			e.printStackTrace();
		}
	}
	
	
	public void run() {
		System.out.println("Socket initiated with: "+socket.getInetAddress().getHostAddress());
		
		boolean listen = true;
		
		while(listen && socket != null){
			 try {
				while(dis != null && dis.available() == 0){
					if (socket.isClosed()){
						stop();
						listen = false;
						break;
					}
					
					try {
						Thread.sleep(5);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				
				long startTreatment = System.currentTimeMillis();
				
				if (dis != null && dos != null) 
					treatPacket(dis, dos);
				
				System.out.println("Treatment delay: "+(System.currentTimeMillis()-startTreatment)+" ms");
				
			} catch (IOException e) {
				System.out.println("Error during packet handling in socket buffer: "+e.getMessage());
				e.printStackTrace();
				stop();
				listen = false;
				
			} 
			
		}
	}
	
	public void treatPacket(DataInputStream dis, DataOutputStream dos) throws IOException{

		long startTreatment = System.currentTimeMillis();
		int requesterType = dis.readByte();
		System.out.println("treatPacket delay start: "+(System.currentTimeMillis()-startTreatment)+" ms");
		
		// who
		switch(requesterType){
			case 0: // Admin request
				doAdmin(dis,dos);
				break;
				
			case 1: // GameMaster
				break;
				
	 		case 2: // Player request
	 			doPlayer(dis,dos);
	 			break;
	 			
	 		case 3: // Sensor request
	 			break;
	 		default:
	 			System.out.println("Bad client type: "+requesterType);
	 			break;	
		}
		
		System.out.println("treatPacket delay end: "+(System.currentTimeMillis()-startTreatment)+" ms");
		
    	// write value on dos
		/*
		 * TODO: test C types correctly encoded for this 
		 * Java server
		 * 
		 * 
		// primitive type
        int i= dis.readInt();
        System.out.println("primitive type: "+i);
        

		 switch (i){
			case 0: 
				boolean b = dis.readBoolean();
				System.out.println("Boolean: "+b);
				dos.writeBoolean(b);
				break;
			case 1:
				byte b0 = dis.readByte();
				System.out.println("Byte: "+b0);
				dos.writeByte(b0);
				break;
			case 2: 
				short s = dis.readShort();
				System.out.println("Short: "+s);
				dos.writeShort(s);
				break;
			case 3:
				int in = dis.readInt();
				System.out.println("Int: "+in);
				dos.writeInt(in);
				break;
			case 4: 
				long l = dis.readLong();
				System.out.println("Long: "+l);
				dos.writeLong(l);
				break;
			case 5: 
				String u = dis.readUTF();
				System.out.println("UTF: "+u);
				dos.writeUTF(u);
				break;
			case 6: 
				float f = dis.readFloat();
				System.out.println("Float: "+f);
				dos.writeFloat(f);
				break;
			case 7: 
				double d = dis.readDouble();
				System.out.println("Double: "+d);
				dos.writeDouble(d);
				break;
		}*/
		
	}

	private static final byte uGASP_FIRSTLOGIN = 1;
	private static final byte uGASP_LOGIN = 2;
	private static final byte uGASP_LOBBY = 3;
	private static final byte uGASP_CREATEAI = 4;
	private static final byte uGASP_CREATEAI_PRIV = 5;
	private static final byte uGASP_JOINAI = 6;
	private static final byte uGASP_JOINAIRND = 7;
	private static final byte uGASP_GETEVENTS = 8;
	private static final byte uGASP_SENDDATA = 9;
	private static final byte uGASP_NAME = 10;
	private static final byte uGASP_STARTAI = 11;
	private static final byte uGASP_ENDAI = 12;
	private static final byte uGASP_QUITAI = 13;
	private static final byte uGASP_QUIT = 14;
	
	private void doPlayer(DataInputStream dis, DataOutputStream dos) {
		try{
			
			long startTreatment = System.currentTimeMillis();
			
	    	// get http request parameters for login 
		    byte mode = dis.readByte();
		    
			// FirstLogin features
			if (mode == uGASP_FIRSTLOGIN){
				try{
					short uid = dis.readShort();
					short appID = dis.readShort(); //Short.parseShort(((String) request.getParameter("appID")));
					String username = dis.readUTF();
					String password = dis.readUTF();
					
					int aID = playerConnector.firstLogin(uid,appID,username,password);

					dos.writeShort((short) aID);
					
				}
				catch(Exception e){
					System.out.println("Exception occured during FirstLogin, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Login features
			else if (mode == uGASP_LOGIN){
				try{
					// get http request parameters for login 
					short aID = dis.readShort(); //Short.parseShort(request.getParameter("aID"));
					String username = dis.readUTF(); //request.getParameter("username");
					String password = dis.readUTF(); //request.getParameter("password");
					String version = dis.readUTF(); //request.getParameter("version");

					int sID = playerConnector.login(aID, username, password);
					dos.writeShort((short) sID);
				}
				catch(Exception e){
					System.out.println("Exception occured during Login, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Lobby features
			else if (mode == uGASP_LOBBY){
				try{
					short sID = dis.readShort();//request.getParameter("sID");
					
			    	playerConnector.lobby(sID, dos);
			    }
				catch(Exception e){
					System.out.println("Exception occured during Lobby, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Name features
			else if (mode == uGASP_NAME){
				try{
			    	// get http request parameters for login
					short aSID =  dis.readShort(); 
					String name = dis.readUTF();

					String pseudo = playerConnector.name(aSID,name);
					dos.writeUTF(pseudo);					
				}
				catch(Exception e){
					System.out.println("Exception occured during Name, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Join features
			else if (mode == uGASP_JOINAI){
				try{
					//get http request parameters for login 
				    short sID =  dis.readShort();//Short.parseShort(request.getParameter("sID"));
					
				    short aIID = dis.readShort(); //request.getParameter("aIID");
					short aSID = playerConnector.joinAI(sID,aIID);
					
					// write the short sID in the http response
					dos.writeShort(aSID);
				}
				catch(Exception e){
					System.out.println("Exception occured during Join, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Create features
			else if (mode == uGASP_CREATEAI){
				try{
					// get http request parameters for login
					short sID = dis.readShort(); //Short.parseShort(request.getParameter("sID"));
					int minActors = dis.readInt();//Integer.parseInt(request.getParameter("minActors"));
					int maxActors = dis.readInt();//Integer.parseInt(request.getParameter("maxActors"));
					
					int aSID = playerConnector.create(sID, minActors, maxActors, new String[]{}); 
					
					dos.writeShort(aSID);
				}
				catch(Exception e){
					System.out.println("Exception occured during Create, may be parameters are invalid: "+e.getMessage());
				}
			}
			// StartAI features
			else if (mode == uGASP_STARTAI){
				try{
					//get http request parameters for login 
					short aSID =  dis.readShort();//Short.parseShort(request.getParameter("aSID"));
					
					boolean ok =playerConnector.start(aSID);
					dos.writeBoolean(ok);
				}
				catch(Exception e){
					System.out.println("Exception occured during StartAI, may be parameters are invalid: "+e.getMessage());
				}
			}
			// InGame features
			else if (mode == uGASP_GETEVENTS || mode == uGASP_SENDDATA){
				try{
					
					short aSID = dis.readShort(); //Short.parseShort(request.getParameter("aSID"));
					
					if (mode == uGASP_SENDDATA)
						playerConnector.sendData(aSID,dis);
					
					// send events to client
					playerConnector.getEvents(aSID,dos);
					
				}
				catch(Exception e){
					System.out.println("Exception occured during InGame, may be parameters are invalid: "+e.getMessage());
				}
			}
			// EndAI features
			else if (mode == uGASP_ENDAI){
				try{
					// get http request parameters for login 
					short aSID =  dis.readShort();//Short.parseShort(request.getParameter("aSID"));
					
					playerConnector.end(aSID);
				}
				catch(Exception e){
					System.out.println("Exception occured during EndAI, may be parameters are invalid: "+e.getMessage());
				}
			}
			// QuitAI features
			else if (mode == uGASP_QUITAI){
				try{
					// get http request parameters for login 
					short aSID = dis.readShort(); // Short.parseShort(request.getParameter("aSID"));
					
					playerConnector.quitAI(aSID);
				}
				catch(Exception e){
					System.out.println("Exception occured during QuitAI, may be parameters are invalid: "+e.getMessage());
				}
			}
			// Quit features
			else if (mode == uGASP_QUIT){
				try{
					// get http request parameters for login 
					short aSID =  dis.readShort();//Short.parseShort(request.getParameter("aSID"));
					
					playerConnector.quit(aSID);
				}
				catch(Exception e){
					System.out.println("Exception occured during Quit, may be parameters are invalid: "+e.getMessage());
				}
			}
			
			dos.flush();
			
			System.out.println("Treatment delay intermediate 4: "+(System.currentTimeMillis()-startTreatment)+" ms");
			
	    }
	    catch (Exception e){
	    	//#ifdef LOGGING
	        //# cat.info("cannot complied to received first login request");
	        //#endif LOGGING
	    }
		
	}

	private void doAdmin(DataInputStream dis, DataOutputStream dos) {
		// TODO Auto-generated method stub
		
	}

	public void stop() {
		
		System.out.println("ClientManager: try to close socket streams");
		thread.interrupt();
		
		try {
			if (dis != null) {dis.close(); dis = null;}
			if (dos != null) {dos.close(); dos = null;}
		} catch (Exception e) {
			e.printStackTrace();
			dis = null;
			dos = null;
		}
		
		try {
			if (socket != null){socket.close(); socket = null;}
		} catch (Exception e) {
			e.printStackTrace();
			socket = null;
		}
		
		System.out.println("ClientManager: end close");
	}


	public String getHostname() {
		return hostname;
	}
}
