/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.impl.core;

//#include parameters.h
////#define LOGGING
////#define DEBUG_CONSOLE
////#define DEBUG_ALERTSCREEN
//#endinclude

//import java.util.Iterator;
import java.util.Vector;

//#ifdef LOGGING
//# import org.apache.log4j.Category;
//#endif LOGGING

import org.ugasp.core.ActorSession;
import org.ugasp.core.Session;
import org.ugasp.eventmodel.event.Event;

/**
 * ActorSession represent the link between a user (represented by instances of Session and Actor)
 * and an ApplicationInstance.
 * It receive and transmit all the events or message in an ApplicationInstance, in game it is the
 * communication link  between the client and the game session server.
 * 
 * @author PELLERIN Romain (pellerin@cnam.fr) - MEGA Project 
 */


public class ActorSessionImpl implements ActorSession {

	private int applicationInstanceID;
	private int actorID;
	private int sessionID;
	private int actorSessionID;
	private String pseudo;
	private Vector events;
	//#ifdef LOGGING
	//# private Category cat;
	//#endif LOGGING
	
	// storage of events must be send to the user represented by the actor session.
    private boolean pseudoModified;

	public ActorSessionImpl(int aIID, Session session){
		applicationInstanceID = aIID;
		actorID = session.getActorID();
		sessionID = session.getSessionID();
		
		
		//#ifdef LOGGING
		//# cat = Category.getInstance("GASPLogging");
		//#endif LOGGING
		
		actorSessionID = PlatformImpl.getPlatform().requestASID();
		
		System.out.println("Create actor session for "+actorID+"/"+sessionID+"/"+actorSessionID);
		
		session.setActorSessionID(actorSessionID);
		events = new Vector();
		
	}

	
	/**
	 * Returns the ApplicationInstance ID linked.
	 * 
	 * @return the ApplicationInstance ID
	 */
	public int getApplicationInstanceID() {
		return applicationInstanceID;
	}
	
	
	/**
	 * Returns the Actor ID of the user.
	 * 
	 * @return the Actor ID
	 */
	public int getActorID() {
		return actorID;
	}

	
	/**
	 * Returns the Session ID of the user
	 * 
	 * @return the Session ID
	 */
	public int getSessionID() {
		return sessionID;
	}
	
	
	/**
	 * Returns the ActorSession ID.
	 * 
	 * @return the ActorSession ID.
	 */
	public int getActorSessionID() {
		return actorSessionID;
	}
	
	
	/**
	 * Returns the current pseudo name of the user session.
	 * 
	 * @return the ActorSession ID
	 */
	public String getPseudoName() {
		return pseudo;
	}
	
	
	/**
	 * Set the current pseudo name of the user session.
	 */
	public void setPseudoName(String name) {
		pseudo = name;
		pseudoModified = true;
	}
	
	
	/**
	 * Tells if the pseudo has modified during the session.
	 */
	public boolean isPseudoHasModified() {
		return pseudoModified;
	}

	
	/**
	 * Return the events stored since the last request from the actor.
	 * 
	 * @return the stack of events
	 */
	public synchronized Vector getEvents(){
		//#ifdef LOGGING
        //# cat.debug("AS("+actorSessionID+"): calls getEvents()");
	    //#endif LOGGING
        
        Vector v = new Vector(events);
	    events.removeAllElements();
	    return purgeEvents(v);
	}
	
	
	/**
	 * Put the event on the stack of the events must be read by the actor.
	 * 
	 * @param e the Event
	 */
	public synchronized void raiseEvent(Event e){
	    events.add(e);
	    //events.insertElementAt(e,events.size());
	    //#ifdef LOGGING
	    //# cat.debug("AS("+actorSessionID+"): event arrived code "+e.getCode()+", index in events "+events.indexOf(e));
	    //#endif LOGGING
	}
	

    /**
     * Purge the deprecated events.
     * 
     * @param v the copy of the events vector
     */
    private Vector purgeEvents(Vector v) {
       return v; 
    }

}
