/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.impl.gameserver;

import java.util.Hashtable;

import org.apache.felix.ipojo.ComponentInstance;
import org.apache.felix.ipojo.ConfigurationException;
import org.apache.felix.ipojo.Factory;
import org.apache.felix.ipojo.InstanceManager;
import org.apache.felix.ipojo.MissingHandlerException;
import org.apache.felix.ipojo.UnacceptableConfiguration;
import org.apache.felix.ipojo.architecture.PropertyDescription;
import org.osgi.framework.BundleContext;
import org.ugasp.gameserver.FactoryManager;

public class FactoryManagerImpl implements FactoryManager{
	// iPojo injection field
	private Factory[] gameServers;
	private BundleContext context;
	
	public FactoryManagerImpl(BundleContext bc) {
		context = bc;
		System.out.println("Factory Manager instanciated context: "+bc);
	}
	
	public InstanceManager newGameServerInstance(int appID){
		try {
			Factory gameServerFactory = getGameServerFactory(appID);
			if (gameServerFactory != null){
					System.out.println("FactoryManagerImpl: factory found corresponding to appID="+appID);
					ComponentInstance componentInstance = gameServerFactory.createComponentInstance(null);
					System.out.println("FactoryManagerImpl: created instance name: "+componentInstance.getInstanceName());
					return (InstanceManager) componentInstance;
			}
			
		} catch (UnacceptableConfiguration e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MissingHandlerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		System.out.println("FactoryManagerImpl: failed to create a game server instance corresponding to appID="+appID);
		return null;
	}
	
	private Factory getGameServerFactory(int appID){
		Factory gameServer = null;
		
		for (int i=0; i<gameServers.length;i++){
			gameServer = gameServers[i];
			PropertyDescription[] properties = gameServer.getComponentDescription().getProperties();	
			PropertyDescription property = null;
			for (int j=0; j<properties.length;j++){
				property = properties[j];
				if (property.getName().equals("appID") && property.getValue().equals(""+appID))
						return gameServer;		
			}
			
		}
		return null;
	}

	public Hashtable getApplicationProperties(int appID) {
		Hashtable appInfos = new Hashtable();
		Factory gameServer = null;
		for (int i=0; i<gameServers.length;i++){
			gameServer = gameServers[i];
			System.out.println("Factory: "+gameServer.getName()+ " "+gameServer.getClassName());
			
			PropertyDescription[] properties = gameServer.getComponentDescription().getProperties();	
			PropertyDescription property = null;
			for (int j=0; j<properties.length;j++){
				property = properties[j];
				System.out.println("property["+j+"]: "+property.getName()+"/"+property.getType()+"/"+property.getValue());
				if (property.getName().equals("appID")){
					appInfos.put(property.getName(), property.getValue());
				}
				else if (property.getName().equals("timeout")){
					appInfos.put(property.getName(), property.getValue());
				}
				else if (property.getName().equals("model")){
					appInfos.put(property.getName(), property.getValue());
				}
			}
			
			Object value = appInfos.get("appID");
			
			if (value != null){
				if (((String) value).equals(""+appID)) return appInfos;
				else appInfos.clear();
			}		
		}
		
		return appInfos;
	}
}
