/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.impl.protocol.osc.illposed;

import java.io.IOException;
import java.net.InetAddress;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;

import org.ugasp.protocol.osc.OSCBundle;
import org.ugasp.protocol.osc.OSCListener;
import org.ugasp.protocol.osc.OSCMessage;
import org.ugasp.protocol.osc.OSCPacket;

/**
 * This class provides OSC protocol functionalities to uGASP bundles,
 * e.g. opening in and out ports with distant address.
 * 
 * 
 * @author PELLERIN Romain (pellerin@cnam.fr) - MEGA Project 
 */
public class OSCConnectorImpl implements org.ugasp.protocol.osc.OSCConnector {
	
	private OSCPortIn portIn;
	private Hashtable portsOut;
	
	public OSCConnectorImpl() throws Exception{
		try{
			portIn = new OSCPortIn(OSCPort.defaultSCOSCPort());
			portIn.startListening();
			portsOut = new Hashtable();
			System.out.println("OSC Connector implementation instantiated");
		}
		catch(Exception e){
			e.printStackTrace();
			System.out.println("OSC Connector> failed to open OSC port IN: "+OSCPort.defaultSCOSCPort());
		}
	}

	public void clean(){
		System.out.println("OSCConnector> closing OSC port in ("+portIn.port+")");
		if (portIn != null) portIn.close();
		
		System.out.println("OSCConnector> OSC port in closed");
		System.out.println("OSCConnector> nb of OSC ports out to close: "+portsOut.size());
		if (portsOut.size() > 0){
			Enumeration e = portsOut.keys();
			while(e.hasMoreElements()){
				((OSCPortOut) portsOut.get(e.nextElement())).close();
			}
			portsOut.clear();
			System.out.println("OSCConnector> all OSC ports out closed");
		}
	}
	
	public void connectTo(InetAddress address, OSCListener listener) throws IOException{
		portIn.addListener(address.getHostAddress(), listener);
		OSCPortOut portOut = new OSCPortOut(address);
		portsOut.put(listener, portOut);
		System.out.println("OSC Connector> "+listener+" connected to "+address.getHostAddress());
	}
	
	public void connectTo(InetAddress address, int port, String oscAddress, OSCListener listener) throws IOException{
		portIn.addListener(address.getHostAddress(), listener);
		OSCPortOut portOut = new OSCPortOut(address,port);
		portsOut.put(listener, portOut);
		System.out.println("OSC Connector> "+listener+" connected to "+address.getHostAddress());
	}

	public void disconnect(OSCListener listener){
		portIn.removeListener(listener);
		
		Enumeration e = portsOut.keys();
		while(e.hasMoreElements()){
			OSCListener l = (OSCListener) e.nextElement();
			if (listener.equals(l)){
				((OSCPortOut) portsOut.get(l)).close();
				portsOut.remove(l);
			}
		}
	}
	
	public void sendPacket(OSCPacket packet, OSCListener listener) throws IOException{
		((OSCPortOut) portsOut.get(listener)).send(packet);
	}

	public OSCBundle newOSCBundle(OSCPacket[] packets, Date timestamp) {
		OSCBundle bundle = new OSCBundleImpl(packets, timestamp);
		return bundle;
	}

	public OSCMessage newOSCMessage(String newAddress, Object[] newArguments) {
		OSCMessage message = new OSCMessageImpl(newAddress, newArguments);
		return message;
	}

	@Override
	public void listenTo(String oscAddress, OSCListener listener)
			throws IOException {
		portIn.addListener(oscAddress, listener);
		System.out.println("OSCConnector: "+listener+" listens to OSC address "+oscAddress);
	}

	@Override
	public void openOutlinkTo(InetAddress ipAddress, int port,
			OSCListener listener) throws IOException {
		OSCPortOut portOut = new OSCPortOut(ipAddress,port);
		portsOut.put(listener, portOut);
	}
}
