/* $Id: OSCPacketDispatcher.java,v 1.1 2009-06-05 10:10:36 pellerin Exp $
 * Created on 28.10.2003
 */
package org.ugasp.impl.protocol.osc.illposed.utility;

import org.ugasp.impl.protocol.osc.illposed.*;
import org.ugasp.protocol.osc.OSCListener;
import org.ugasp.protocol.osc.OSCMessage;

import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * @author cramakrishnan
 *
 * Copyright (C) 2003, C. Ramakrishnan / Auracle
 * All rights reserved.
 * 
 * See license.txt (or license.rtf) for license information.
 * 
 * Dispatches OSCMessages to registered listeners.
 * 
 */

public class OSCPacketDispatcher {
	// use Hashtable for JDK1.1 compatability
	private Hashtable listeners = new Hashtable();
	
	/**
	 * 
	 */
	public OSCPacketDispatcher() {
		super();
	}

	public void addListener(String address, OSCListener listener) {
		listeners.put(listener,address); // uGASP mod: params inversion to support different listener for a same address 
	}
	
	// uGASP add: OSC connector is a permanent service
	// so we have to remove destroyed OSCListener instances
	public void removeListener(OSCListener listener) {
		Enumeration e = listeners.keys();
		while(e.hasMoreElements()){
			OSCListener l = (OSCListener) e.nextElement();
			if (listener.equals(l)){
				listeners.remove(l);
			}
		}
	}
	
	public void dispatchPacket(OSCPacketImpl packet) {
		if (packet instanceof OSCBundleImpl)
			dispatchBundle((OSCBundleImpl) packet);
		else
			dispatchMessage((OSCMessageImpl) packet);
	}
	
	public void dispatchPacket(OSCPacketImpl packet, Date timestamp) {
		if (packet instanceof OSCBundleImpl)
			dispatchBundle((OSCBundleImpl) packet);
		else
			dispatchMessage((OSCMessageImpl) packet, timestamp);
	}
	
	private void dispatchBundle(OSCBundleImpl bundle) {
		Date timestamp = bundle.getTimestamp();
		OSCPacketImpl[] packets = bundle.getPackets();
		for (int i = 0; i < packets.length; i++) {
			dispatchPacket(packets[i], timestamp);
		}
	}
	
	private void dispatchMessage(OSCMessageImpl message) {
		dispatchMessage(message, null);
	}
	
	private void dispatchMessage(OSCMessageImpl message, Date time) {
		Enumeration keys = listeners.keys();
		while (keys.hasMoreElements()) {
			// uGASP mod: params inversion to support different listener for a same address
			OSCListener l = (OSCListener) keys.nextElement();
			System.out.println("OSCPacketDispatcher> dispatch OSC msg search listeners: "+l);
			
			String address = (String) listeners.get(l);
			System.out.println("OSCPacketDispatcher> OSC message received address: listens to OSC address: "+message.getAddress());
			
			if (message.getAddress().startsWith(address)) {
				System.out.println("OSCPacketDispatcher> forward the packet to the listener");
				l.acceptMessage(time, (OSCMessage) message);
			}
		}
	}

	public int getNbOfListeners() {
		return listeners.size();
	}
}
