/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.impl.service.external.lobby;

//#include parameters.h
////#define LOGGING
////#define DEBUG_CONSOLE
////#define DEBUG_ALERTSCREEN
//#endinclude

import java.util.Iterator;
import java.util.Vector;

//#ifdef LOGGING
//# import org.apache.log4j.Category;
//#endif LOGGING

import org.ugasp.core.ApplicationInstance;
import org.ugasp.core.MasterApplicationInstance;
import org.ugasp.core.Platform;
import org.ugasp.service.external.lobby.Lobby;

/**
 * Lobby provide informations to the users on the current state of the Gaming Platform.
 * 
 * @author PELLERIN Romain (pellerin@cnam.fr) - MEGA Project 
 */
public class LobbyImpl implements Lobby {
	
	//iPojo injection field
	private Platform core;
	
	//#ifdef LOGGING
    //# private Category cat;
    //#endif LOGGING
    
	public LobbyImpl(){System.out.println("Lobby Service instantiated");}
	
	
	/**
	 * Returns a list of the current active ApplicationInstances associated to a specific application
	 * with the ActorSession contains in fact a stream : (ApplicationInstanceID (username,ActorID)*)*
	 * 
	 * @param sID the application ID
	 * @return a stream
	 */
	public Vector getApplicationInstancesInfos(int sID) {
		//#ifdef LOGGING
	    //# cat = Category.getInstance("GASPLogging");
	    //#endif LOGGING
	    Vector applicationInstancesInfos = new Vector();
		MasterApplicationInstance masterApp =  core.getSessionOwner(sID);
		if (masterApp != null){
			for(Iterator iter = masterApp.enumerateApplicationInstance(); iter.hasNext();){
				ApplicationInstance appInstance = (ApplicationInstance) iter.next();
				if (appInstance.isJoinable())
				    applicationInstancesInfos.add(appInstance.getApplicationInstanceInfos());
			}
		}
		
		return applicationInstancesInfos;
	}

}
