/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.impl.service.external.player;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import org.ugasp.core.Actor;
import org.ugasp.core.ActorSession;
import org.ugasp.core.ApplicationInstance;
import org.ugasp.core.MasterApplicationInstance;
import org.ugasp.core.Platform;
import org.ugasp.core.Session;
import org.ugasp.core.utils.ApplicationInstanceInfos;
import org.ugasp.eventmodel.event.DataEvent;
import org.ugasp.eventmodel.event.EndEvent;
import org.ugasp.eventmodel.event.Event;
import org.ugasp.eventmodel.event.JoinEvent;
import org.ugasp.eventmodel.event.QuitEvent;
import org.ugasp.eventmodel.event.StartEvent;
import org.ugasp.eventmodel.event.impl.DataEventImpl;
import org.ugasp.eventmodel.event.impl.EndEventImpl;
import org.ugasp.eventmodel.event.impl.QuitEventImpl;
import org.ugasp.eventmodel.event.impl.StartEventImpl;
import org.ugasp.protocol.moods.CustomTypes;
import org.ugasp.service.external.lobby.Lobby;
import org.ugasp.service.external.player.PlayerConnector;

/**
 * When the game programmer wants to use his custom types classes,
 * first he must describe his custom types in the types.xml with the
 * correct syntax. 
 * Then he must run the custom types generator in org.mega.moods package
 * with the command line: MoodsGenerator types.xml packageName
 * The generator generate a file CustomTypes.java that is the encoder/decoder
 * of the game custom types.
 * 
 * 
 * @author PELLERIN Romain (pellerin@cnam.fr) - MEGA Project 
 */
public class PlayerConnectorImpl implements PlayerConnector{
    
	//iPojo dynamic binding var
	private Platform core;
	private Lobby lobby;
	
	public PlayerConnectorImpl(){
		System.out.println("Player Connector instantiated");
	}
	
	@Override
	public int create(short sID, int minActors, int maxActors, String[] actors) {
		if (minActors > 0 && minActors<= maxActors){
			// create an application instance owned by the sid represented actor
			// with necessited parameters minActors and maxActors 
			// if the table actors is empty then the application instance
			// is private statued else public
			return core.createAI(Integer.parseInt(sID+""),minActors,maxActors,actors);
		
			// write the short sID in the http response
		}
		else{
			//#ifdef LOGGING
		    //# cat.debug("invalid minActors and maxActors parameters in create request ("+minActors+","+maxActors+")");
		    //#endif LOGGING
		    
		    return 0;
		}
	}


	@Override
	public void end(short aSID) {
		EndEvent ee = new EndEventImpl(aSID);
		
		ApplicationInstance appIns = core.getActorSessionOwner(aSID);
		
		//#ifdef LOGGING
		//# cat.debug("EndAI: send EndEvent from aSID n�"+aSID+ "to AI("+appIns.getApplicationInstanceID()+")");
		//#endif LOGGING
		
		appIns.onEndEvent(ee);
	}


	@Override
	public int firstLogin(short uid, short appID, String username, String password) {
		// registration of the user with his choice of username and password
		// return a short aID, permanent actorID associated with the userID and applicationID
		// or the short "0" if the user have not the appropriate rights
		return core.getDBManager().registerNewActor(""+uid,appID,username,password);
	}


	@Override
	public void sendData(short aSID, String data) {
		// take the platform, the AI , the MAI and the Session associated to
		// ASID
		ApplicationInstance appIns = core.getActorSessionOwner(aSID);

		// take the specific custom types for the application
		org.ugasp.protocol.moods.string.CustomTypes customTypes = appIns.getCustomTypes4StringEncoding();

		Hashtable h = customTypes.decodeData(data);

		if (h.size() != 0) {
			//#ifdef LOGGING
			//# cat.debug("Received data from client, number of objects: "
					//# + h.size());
			//#endif LOGGING
			DataEvent de = new DataEventImpl(aSID, h);
			appIns.onDataEvent(de);
		}
	}
	
/*TODO: FLASH

	@Override
	public String getEvents(short aSID) {
		ApplicationInstance appIns = core.getActorSessionOwner(aSID);
		ActorSession aSession = appIns.getActorSession(aSID);
		int sID = aSession.getSessionID();
		MasterApplicationInstance masterApp = core.getSessionOwner(sID);
		
		String data = encodeEvents4Flash(appIns.getActorSession(aSID).getEvents(), appIns.getCustomTypes4StringEncoding());
		
		// destroy the actor session linked to aSID
		// and if all actor sessions are destroyed calling the owner master
		// app
		// to destroy the application instance.
		if (appIns.isDestroyable()) {
			QuitEventImpl qe = new QuitEventImpl(aSID);
			appIns.onQuitEvent(qe);
			if (appIns.isActorSessionEmpty())
				masterApp.removeApplicationInstance(appIns.getApplicationInstanceID());
		}
		
		return data;
	}
	// specific flash encoding parsing methods
	 private String addEncodingSpacer(Object o) {
		 return new String(o+"|");
	 }
	 
	 private String encodeEvents4Flash(Vector events, org.ugasp.core.moods.string.CustomTypes customTypes){
       String data = "";
       data += addEncodingSpacer(""+events.size());
       for(Iterator iter = events.iterator(); iter.hasNext();){
           Event e = (Event) iter.next();
           int code = e.getCode();
           data += addEncodingSpacer(""+code);
           switch (code){
           	case 1: // JoinEvent
           	    JoinEvent je = (JoinEvent) e;
           	    data += addEncodingSpacer(""+je.getActorSessionID());
           	    data += addEncodingSpacer(je.getUsername());
           	    break;
           	case 2: // StartEvent
           	    StartEvent se = (StartEvent) e;
           	    data += addEncodingSpacer(""+se.getActorSessionID());
           	    break;
           	case 3:
           	    EndEvent ee = (EndEvent) e;
           	    data += addEncodingSpacer(""+ee.getActorSessionID());
           	    break;
           	case 4:
           	    QuitEvent qe = (QuitEvent) e;
           	    data += addEncodingSpacer(""+qe.getActorSessionID());
           	    break;
           	case 5:
           	    DataEvent de = (DataEvent) e;
           	    
           	    //#ifdef LOGGING
           	    //# cat.debug("InGame: start to encoding DataEvent("+e.getActorSessionID()+","+de.getData().size()+")");
           	    //#endif LOGGING
           	    
           	    data += addEncodingSpacer(""+de.getActorSessionID());
           	    data += customTypes.encodeData(de.getData());
           	    break;
           	default: break;
           }
       }
       return data;
	}
*/

	@Override
	public void sendData(short aSID, DataInputStream dis) {
		try{
			// take the platform, the AI , the MAI and the Session associated to ASID
			ApplicationInstance appIns = core.getActorSessionOwner(aSID);

			CustomTypes customTypes = appIns.getCustomTypes();
			
			System.out.println("customTypes="+customTypes);
			
	        Hashtable h = customTypes.decodeData(dis);
	
	        if (h.size()!=0){
	        	//#ifdef LOGGING
	        	//# cat.debug("Received data from client, number of objects: "+h.size());
	            //#endif LOGGING

	        	DataEvent de = new DataEventImpl(aSID,h);
	            appIns.onDataEvent(de);
	        }
		}
		catch(Exception e){e.printStackTrace();};
	}


	@Override
	public void getEvents(short aSID, DataOutputStream dos) {
		try{
			ApplicationInstance appIns = core.getActorSessionOwner(aSID);
			ActorSession aSession = appIns.getActorSession(aSID);
			int sID = aSession.getSessionID();
			MasterApplicationInstance masterApp = core.getSessionOwner(sID);
			
		    int i = encodeEvents(appIns.getActorSession(aSID).getEvents(),dos, appIns.getCustomTypes());
	        
		    // destroy the actor session linked to aSID
		    // and if all actor sessions are destroyed calling the owner master app
		    // to destroy the application instance.
		    if (appIns.isDestroyable()){
		    	QuitEventImpl qe = new QuitEventImpl(aSID);
	            appIns.onQuitEvent(qe);  
	            if (appIns.isActorSessionEmpty())	
	               masterApp.removeApplicationInstance(appIns.getApplicationInstanceID());
	        }
		}
		catch(Exception e){e.printStackTrace();}
	}
	
	/**
     * @param events
     * @throws Exception
     */
    private int encodeEvents(Vector events,DataOutputStream dos,CustomTypes customTypes) throws Exception {
        dos.writeByte(events.size());
        for(Iterator iter = events.iterator(); iter.hasNext();){
            Event e = (Event) iter.next();
            int code = e.getCode();
            dos.writeByte(code);

            switch (code){
            	case 1: // JoinEvent
            	    JoinEvent je = (JoinEvent) e;
            	    dos.writeShort(je.getActorSessionID());
            	    dos.writeUTF(je.getUsername());
            	    break;
            	case 2: // StartEvent
            	    StartEvent se = (StartEvent) e;
            	    dos.writeShort(se.getActorSessionID());
            	    break;
            	case 3:
            	    EndEvent ee = (EndEvent) e;
            	    dos.writeShort(ee.getActorSessionID());
            	    break;
            	case 4:
            	    QuitEvent qe = (QuitEvent) e;
            	    dos.writeShort(qe.getActorSessionID());
            	    break;
            	case 5:
            	    DataEvent de = (DataEvent) e;
            	    
            	    //#ifdef LOGGING
            	    //# cat.debug("InGame: start to encoding DataEvent("+e.getActorSessionID()+","+de.getData().size()+")");
            	    //#endif LOGGING
            	    
            	    dos.writeShort(de.getActorSessionID());
            	    customTypes.encodeData(de.getData(),dos);
            	    break;
            	default: break;
            }
        }
        return events.size();
        
    }


	@Override
	public short joinAI(short sID, short aIID) {
		short aSID;
		
		if (aIID == 0){ 
			//Join Randomly an Application Instance
			
			//#ifdef LOGGING
		    //# cat.debug("JoinServlet: join random");
			//#endif LOGGING
		    
		    aSID = (short) core.joinAIRnd(sID);
		}
		else{
			// Join the specific Application Instance
			
			//#ifdef LOGGING
		    //# cat.debug("JoinServlet: join AI("+applicationInstanceID+")");
			//#endif LOGGING
		    
			aSID = (short) core.joinAI(sID,aIID);
		}
		
		return aSID;
	}

/* TODO:
	@Override
	public String lobby(String sID) {
		// requiring the lobby infos to the platform for the application
		// associated with the sID
		Vector lobbyInfos = core.getLobby().getApplicationInstancesInfos(
				Integer.parseInt(sID));

		// encoded string to be sent to Flash clients
		String data = "";
			
		// push in the DataOutputStream the number of application instances
		data += addEncodingSpacer(""+lobbyInfos.size());

		for (Iterator iter = lobbyInfos.iterator(); iter.hasNext();) {
			ApplicationInstanceInfos appInstanceInfos = (ApplicationInstanceInfos) iter
					.next();

			// push the aIID of the current application instance
			data += addEncodingSpacer(""+appInstanceInfos.getApplicationInstanceID());

			// push the minimum starting number of actors
			data += addEncodingSpacer(""+appInstanceInfos.getMinActors());

			// push the number of actors currently connected in
			data += addEncodingSpacer(""+appInstanceInfos.getNbPlayers());

			// push the maximum number of actors
			data += addEncodingSpacer(""+appInstanceInfos.getMaxActors());

			for (Iterator iter2 = appInstanceInfos.getPlayerList().iterator(); iter2
					.hasNext();) {
				String username = (String) iter2.next();
				Integer actorID = (Integer) iter2.next();

				// push the username and the actor ID of the current actor
				data += addEncodingSpacer(username);
				data += addEncodingSpacer(""+actorID.intValue());
			}
		}
		
		return data;
	}
*/

	@Override
	public void lobby(short sID, DataOutputStream dos) {
		try{
			// requiring the lobby infos to the platform for the application associated with the sID
			Vector lobbyInfos = lobby.getApplicationInstancesInfos((int) sID);
			
			// push in the DataOutputStream the number of application instances
			dos.writeShort(lobbyInfos.size());
			
			for (Iterator iter = lobbyInfos.iterator(); iter.hasNext();){
				ApplicationInstanceInfos appInstanceInfos = (ApplicationInstanceInfos) iter.next();
				
				// push the aIID of the current application instance
				dos.writeShort(appInstanceInfos.getApplicationInstanceID());
				
				// push the minimum starting number of actors
				dos.writeShort(appInstanceInfos.getMinActors());
				
				
				// push the number of actors currently connected in
				dos.writeShort(appInstanceInfos.getNbPlayers());
				
				// push the maximum number of actors 
				dos.writeShort(appInstanceInfos.getMaxActors());
				
				for (Iterator iter2 = appInstanceInfos.getPlayerList().iterator(); iter2.hasNext();){
					String username = (String) iter2.next();
					Integer actorID = (Integer) iter2.next();
					
					// push the username and the actor ID of the current actor
					dos.writeUTF(username);
					dos.writeShort(actorID.intValue());
				}
			}
		}
		catch (Exception e){e.printStackTrace();}
	}


	@Override
	public int login(short aID, String username, String password) {
		// login, return a short representing the sessionID resulting from the
		// session instanciation
		// or the short "0" if the authentification informations are not valid
		return core.login(aID, username, password);
	}


	@Override
	public String name(short aSID, String name) {

		ApplicationInstance appIns = core.getActorSessionOwner(aSID);
		ActorSession aS = appIns.getActorSession(aSID);
		MasterApplicationInstance masterApp = core.getSessionOwner(aS
				.getSessionID());
		Actor actor = masterApp.getActor(aS.getActorID());

		// treat the new pseudoname
		aS.setPseudoName(appIns.treatPseudo(name));
		actor.setPseudoName(aS.getPseudoName());
		core.getDBManager().saveLastUsedPseudo(actor.getActorID(),
				actor.getPseudoName());

		// raise the new pseudo to all other players
		appIns.raisePseudoModification(aS.getActorSessionID(), aS
				.getPseudoName());

		if (!name.equals(aS.getPseudoName()))
			aS.getPseudoName();
		
		return aS.getPseudoName();
	}


	@Override
	public void quit(int sID) {
		core.quit(sID);
	}


	@Override
	public void quitAI(short aSID) {
		// create quit event
		QuitEvent qe = new QuitEventImpl(aSID);
		
		// take the platform, the AI , the MAI and the Session associated to ASID
		ApplicationInstance appIns = core.getActorSessionOwner(aSID);
		ActorSession aS = appIns.getActorSession(aSID);
		int sID = aS.getSessionID();
		
		MasterApplicationInstance masterApp = core.getSessionOwner(sID);
		Session session = masterApp.getSession(sID);
			
		// raise event to AI
		appIns.onQuitEvent(qe);
		session.setActorSessionID(0);
	}


	@Override
	public boolean start(short aSID) {
		StartEvent se = new StartEventImpl(aSID);
		ApplicationInstance appIns = core.getActorSessionOwner(aSID);
		boolean b = false;
		if ((appIns.getOwnerAID()==aSID) && appIns.isStartable()){
			//#ifdef LOGGING
			//# cat.debug("StartAI: send StartEvent from aSID n�"+aSID+ " to AI("+appIns.getApplicationInstanceID()+")");
			//#endif LOGGING
			
			appIns.onStartEvent(se);
			b = true;
		}
		
		return b;
	}

	@Override
	public String getEvents(short asid) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public String lobby(short sid) {
		// TODO Auto-generated method stub
		return null;
	}
}
