/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.impl.service.internal.gameserver.location;


import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;

import org.ugasp.service.internal.gameserver.location.MapProvider;

public class MapProviderImpl implements MapProvider{

	public MapProviderImpl(){
		System.out.println("YahooMaps based map provider instanciated!");
	}

	/**
	 * Gets the map from a map provider and returns the map data as a byte array.
	 * This implementation aims the YahooMaps ImageAPI map provider.
	 * 
	 * @param devID - developer ID (often required by map providers)
	 * @param lat - GPS latitude
	 * @param lon - GPS longitude
	 * @param screen_width
	 * @param screen_height
	 * @param zoom - map zoom
	 * @return map data
	 */
	public byte[] getMap(String devID, double lat, double lon, int screen_width, int screen_height, int zoom) {
		try{
    		String mapUrl = httpMapRequest(devID,lat,lon,zoom,screen_width,screen_height);

    		byte[] data = httpMapContentRequest(mapUrl);
    		
    		if (data != null){
    			System.out.println("map data length:"+data.length);
    		}
    		
    		return  (data == null? new byte[]{} : data);
    	}
    	catch (Exception e){
    		System.out.println("Error during map request:"+e.getMessage());
    		return new byte[]{};
    	}
        finally {try{connection.disconnect();} catch (Exception e){connection = null;}}
	}
	
	
	private HttpURLConnection connection;
	private static String mapProviderURL = "http://api.local.yahoo.com/MapsService/V1/mapImage";
	
	static private DataOutputStream dos = null;
	static private DataInputStream dis = null;
	
	/**
	 * Retrieves the map image url from the map provider (YahooMaps)
	 * 
	 * @param servletName the name of the servlet
	 * @param formData the HTTP post parameters 
	 * @return the DataInputStream
	 * @throws IOException
	 */
    private String httpMapRequest(String devID,double lat, double lon, int zoom, int width, int height) throws IOException{
    	String formData = "?"
    						+"appid="
    						+devID
    						+"&latitude="+lat
    						+"&longitude="+lon
    						+"&zoom="+zoom
    						+"&image_width="+width 
    						+"&image_height="+height;
    	
    	System.out.println("MapProvider> request URL:"+mapProviderURL+formData);
    	
    	//#ifdef DEBUG_CONSOLE_MAP
    	//# System.out.println("URL:"+mapProviderURL+formData);
    	//#endif DEBUG_CONSOLE_MAP
        
        URL yahoomaps = new URL(mapProviderURL+formData);
        connection = (HttpURLConnection) yahoomaps.openConnection();
        connection.setRequestMethod("GET");
		connection.setRequestProperty("Accept", "text/xml");
        connection.connect();
		
		if (connection.getResponseCode() == HttpURLConnection.HTTP_OK){;

			//#ifdef DEBUG_CONSOLE_MAP
			//# System.out.println("content type: "+connection.getType()+"-"+connection.getHeaderFieldInt(Arg.CONTENT_LENGTH, 0));
			//# System.out.println("content length: "+connection.getLength()+"-"+connection.getHeaderFieldInt(Arg.CONTENT_LENGTH, 0));
			//#endif DEBUG_CONSOLE_MAP
			
    		dis = new DataInputStream(connection.getInputStream());
    		String s = "";
    		
    		int index = 0;
    	  	while(true){
	    		byte b = (byte) dis.read();
	    		if (b == -1) break;
	    		
	    		s += (char) b;
	    		//data[index] = b;
	    		index++;
	    	}

    	  	// mini parser of result content
    	  	// index of '<'Result'>
    	  	boolean resultFound = false;
    	  	int _index = 0;
    	  	int count = 0;
    	  	
    	  	while (!resultFound){

    	  		int indexOf = s.indexOf('<',_index);
    	  		if (indexOf == -1) break;
    	  		
    	  		String sub = s.substring(indexOf+1, indexOf+1+6);
    	  		
    	  		//#ifdef DEBUG_CONSOLE_MAP
    	  		//# System.out.println("is Result ? "+ sub);
    	  		//#endif DEBUG_CONSOLE_MAP
    	  		
    	  		resultFound = sub.equals("Result");
    	  		
    	  		_index = s.indexOf('>',_index)+1; //>_
    	  		count++;
    	  	}

    	  	// get <Result> url content
    	  	int end = s.indexOf('<', _index);
    	  	String resultUrl = s.substring(_index,end);
    	  	
    	  	//#ifdef DEBUG_CONSOLE_MAP
    	  	//# System.out.println("url:"+resultUrl);
			//# System.out.println("http ok");
			//#endif DEBUG_CONSOLE_MAP
			
    	  	connection.disconnect();

			return resultUrl;
		}
		
		else {
			return "";
		}
	}
    
    
    /**
	 * Gets the map from the map provider (YahooMaps)
	 * 
	 * @param servletName the name of the servlet
	 * @param formData the HTTP post parameters 
	 * @return the DataInputStream
	 * @throws IOException
	 */
    private byte[] httpMapContentRequest(String mapUrl) throws IOException{
    	URL yahoomaps = new URL(mapUrl);
        connection = (HttpURLConnection) yahoomaps.openConnection();
        connection.setRequestMethod("GET");
		connection.setRequestProperty("Accept", "image/png");
		connection.connect();
		
		if (connection.getResponseCode() == HttpURLConnection.HTTP_OK){

			System.out.println("9");
			
			//#ifdef DEBUG_CONSOLE_MAP
			//# System.out.println("content type: "+connection.getType()+"-"+connection.getHeaderFieldInt(Arg.CONTENT_LENGTH, 0));
			//# System.out.println("content length: "+connection.getLength()+"-"+connection.getHeaderFieldInt(Arg.CONTENT_LENGTH, 0));
			//#endif DEBUG_CONSOLE_MAP
			
			//byte[] data = new byte[connection.getContentLength()];
			
			
			DataInputStream dis = new DataInputStream(connection.getInputStream());
			
			
			// no content length in Yahoo maps so we use a byte array output stream
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			
			int i = 0;
			while( (i=dis.read()) != -1){
				baos.write(i);
			}
			
			byte[] data = baos.toByteArray();
			
			baos.close();
			dis.close();
			
			System.out.println("data content length:"+data.length);
		
			//dis.readFully(data);	

			System.out.println("10");
			
			connection.disconnect();
			return data;
		}
		else {

			System.out.println("11");
			return null;
		}
	}
	
	
}
