/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.impl.service.internal.gameserver.location;

import java.awt.Point;

import org.ugasp.impl.service.internal.gameserver.location.openmap.LLXY;
import org.ugasp.impl.service.internal.gameserver.location.openmap.Length;
import org.ugasp.service.internal.gameserver.location.LatLonPoint;
import org.ugasp.service.internal.gameserver.location.MapProvider;
import org.ugasp.service.internal.gameserver.location.PrecisePoint;
import org.ugasp.service.internal.gameserver.location.Projection;

public class ProjectionImpl implements Projection{

	private MapProvider mapProvider = new MapProviderImpl();
	
	public ProjectionImpl(){
		System.out.println("Location Projection instanciated!");
		
	}
	
	private static final int[] YAHOO_ZOOM_SCALE = new int[]{ 4625, 9250, 18500, 37000, 74000, 148000,
		 296000, 592000, 0, 0, 0, 0,
		 0, 0, 0, 0, 0 };

	private LatLonPointImpl center;
	private double latCenter;
	private LLXY ll2XYProjection;
	
	public byte[] getMap(String devID, double lat, double lon, int screen_width, int screen_height, int zoom) {
		return mapProvider.getMap(devID, lat, lon, screen_width, screen_height, zoom);
	}
	
	public void initLL2XY(double lat, double lon, int screen_width, int screen_height, int zoom){
		latCenter = lat;
		center = new LatLonPointImpl(lat, lon);
		ll2XYProjection = new LLXY(center,YAHOO_ZOOM_SCALE[zoom-1],screen_width,screen_height);
	}
	
	public PrecisePoint LL2XYProjectionPrecisely(double lat, double lon){
		PrecisePoint p = new PrecisePointImpl();
		return ll2XYProjection.forwardPrecisely((float)lat, (float)lon, p);
	}
	
	public Point LL2XYProjection(double lat, double lon){
		Point p = new Point();
		return ll2XYProjection.forward((float)lat, (float)lon, p);
	}
	
	public double errorAdjustement(double latCoord){
		return latCoord +(latCoord-latCenter)*0.53;
	}
	
	public double computeDistance(double lat1, double lon1, double lat2, double lon2){
		LatLonPointImpl pos1 = new LatLonPointImpl(lat1, lon1);
		LatLonPointImpl pos2 = new LatLonPointImpl(lat2, lon2);
		
		double distance = pos1.distance(pos2);
		distance = Length.get("meter").fromRadians(distance);
		
		return distance;
	}
	
	public LatLonPoint newLatLonPoint(
			double lat, double lon) {
		return new LatLonPointImpl(lat,lon);
	}
}
