//#include parameters.h
////#define DEBUG_CONSOLE
////#define STORE_DATA
//#endinclude
package org.ugasp.impl.service.internal.gameserver.rfid;

import java.net.*;
import java.util.Hashtable;
import java.util.Vector;
import java.io.*;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

import org.xml.sax.*;
import org.xml.sax.ContentHandler;
import org.xml.sax.helpers.DefaultHandler;

class SAXReader extends DefaultHandler{
	public static Vector mimeObjects = new Vector();
	public String resourcesDirectory;
	private SAXParserFactory factory;

	public SAXReader(){
		try{
		   // Use the default (non-validating) parser
	       factory = SAXParserFactory.newInstance();
		 }
		 catch(Throwable t){
		     System.out.println("Error during SAXReader instantiation: "+t.getMessage());
		     t.printStackTrace();
		 }
	}
	
	// loading step
	private Hashtable temp;
	private Object monitor = new Object();
	
	public synchronized Hashtable loadRFIDData(InputStream xml) throws ParserConfigurationException, SAXException, IOException {
		temp = new Hashtable();

		// Parse the input
		SAXParser saxParser = factory.newSAXParser();
		readingStep = -1;
		saxParser.parse(xml, this);

		return temp;
	}
	
	private RFIDDataImpl tagData;

	// reading step
	private static byte readingStep = -1;
	private static final byte ROOT = 0;
	private static final byte TAG_ROOT = 1;
	private static final byte CONTEXT_LOCATION = 2;
	private static final byte CONTEXT_GAMEEVENTS = 3;
	private static final byte CONTEXT_GAMEEVENT = 4;
	private static final byte CONTEXT_GAMEOBJECTS = 5;
	private static final byte CONTEXT_GAMEOBJECT = 6;
	private static final byte CONTEXT_CUSTOMINFO = 7;
	private static final byte CONTEXT_CUSTOMINFO_DATA = 8;
	
	public void startElement(String namespaceURI, String sName, // simple name
			String qName, // qualified name
			Attributes attrs) throws SAXException {
		
		if(qName.equals("tags")){
			if (readingStep == -1){
				readingStep = ROOT;
			}
		}
		else if(qName.equals("tag")){
			if (readingStep == ROOT){
				readingStep = TAG_ROOT;
			
				tagData = new RFIDDataImpl();
				tagData.UID = attrs.getValue("uid");
			}
		}
		else if (qName.equals("location")){
			if (readingStep == TAG_ROOT){
				readingStep = CONTEXT_LOCATION;
				return;
			}
		}
		else if (qName.equals("gps")){
			if (readingStep == CONTEXT_LOCATION){
				if (tagData.locationType == tagData.LOCATION_TYPE_VIRTUAL)
					tagData.locationType = tagData.LOCATION_TYPE_BOTH;
				else tagData.locationType = tagData.LOCATION_TYPE_GPS;
				
				tagData.latitude = Double.parseDouble(attrs.getValue("latitude"));
				tagData.longitude = Double.parseDouble(attrs.getValue("longitude"));
			}
		}
		else if (qName.equals("custom")){
			if (readingStep == CONTEXT_LOCATION){
				if (tagData.locationType == tagData.LOCATION_TYPE_VIRTUAL)
					tagData.locationType = tagData.LOCATION_TYPE_BOTH;
				else tagData.locationType = tagData.LOCATION_TYPE_GPS;
				
				for (int i=0; i< attrs.getLength(); i++)
					tagData.customLocation.put(attrs.getQName(i), attrs.getValue(i));
			}
		}
		else if(qName.equals("gameEvents")){
			if (readingStep == TAG_ROOT){
				readingStep = CONTEXT_GAMEEVENTS;
			}
		
		}
		else if(qName.equals("gameEvent")){
			if (readingStep == CONTEXT_GAMEEVENTS){
				readingStep = CONTEXT_GAMEEVENT;
				tagData.gameEvents.put(attrs.getValue("id"), attrs.getValue("value")); 
			    //new Hashtable() /*values*/); 
				//TODO: values else use only a vector
			}
		}
		else if (qName.equals("gameObjects")){
			if (readingStep == TAG_ROOT){
				readingStep = CONTEXT_GAMEOBJECTS;
			}
		}
		else if (qName.equals("gameObject")){
			if (readingStep == CONTEXT_GAMEOBJECTS){
				readingStep = CONTEXT_GAMEOBJECT;
				tagData.gameObjects.put(attrs.getValue("id"), new Hashtable() /*values*/);
				//TODO: values else use only a vector
			}
		}
		else if (qName.equals("customData")){
			if (readingStep == TAG_ROOT){
				readingStep = CONTEXT_CUSTOMINFO;
				tagData.gameObjects.put(attrs.getValue("id"), new Hashtable() /*values*/); 
			}
		}
		else if (qName.equals("data")){
			if (readingStep == CONTEXT_CUSTOMINFO){
				readingStep = CONTEXT_CUSTOMINFO_DATA;
				tagData.gameObjects.put(attrs.getValue("name"), new Hashtable() /*values*/);
				//TODO: values else only one value
			}
		}
		
	}
	
	
	public void endElement(String namespaceURI, String localName, String qName)  throws SAXException {
		if(qName.equals("tags")){
			readingStep = -1;
		}
		else if(qName.equals("tag")){
			temp.put(tagData.getUID(),tagData);
			readingStep = ROOT;
		}
		else if (qName.equals("location") || qName.equals("gameEvents") || qName.equals("gameObjects") || qName.equals("customData")){
			readingStep = TAG_ROOT;
		}
		else if (qName.equals("gameEvent")){
			readingStep = CONTEXT_GAMEEVENTS;
		}
		else if (qName.equals("gameObject")){
			readingStep = CONTEXT_GAMEOBJECTS;
		}
		else if (qName.equals("data")){
			if (readingStep == CONTEXT_CUSTOMINFO_DATA ) readingStep = CONTEXT_CUSTOMINFO;
		}
	}
}

