/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.impl.service.internal.gameserver.rfid;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.ugasp.service.internal.gameserver.rfid.TagManager;
import org.ugasp.service.internal.gameserver.rfid.RFIDData;

public class TagManagerImpl implements TagManager{

	private SAXReader saxReader;
	private Hashtable rfidDataPool;
	
	public TagManagerImpl(){
		saxReader = new SAXReader();
		rfidDataPool = new Hashtable();
		System.out.println("RFID TAG Manager instanciated!");
	}

	@Override
	public RFIDData getData(int appID, String uid) {
		Hashtable appRFIDData = (Hashtable) rfidDataPool.get(""+appID);
		if (appRFIDData != null){
			return (RFIDData) appRFIDData.get(uid);
		}
		return null;
	}
	
	@Override
	public boolean loadRFIDData(int appID, InputStream xmlRFIDData){
		try{
			// avoid to load data for each game instance
			if (rfidDataPool.containsKey(""+appID)) return true;
			
			// load RFID data
			Hashtable tagsData = saxReader.loadRFIDData(xmlRFIDData);
			rfidDataPool.put(""+appID, tagsData);
			System.out.println("RFID data loaded for application "+appID+":");
			return true;
		}
		catch (Exception e){
			System.out.println("Could not load RFID data for application "+appID+":"+e.getMessage());
			e.printStackTrace();
		}
		return false;
	}

	@Override
	public String[] getRFIDTagList(int appID) {
		Hashtable appRFIDData = (Hashtable) rfidDataPool.get(""+appID);
		
		String[] uids = null;
		if (appRFIDData != null){
			uids = new String[appRFIDData.size()];
			Enumeration e = appRFIDData.keys();
			int index = 0;
			while (e.hasMoreElements()){
				String uid = (String) e.nextElement();
				uids[index] = uid;
				index++; 
			}
		}
		return uids;
	}
}
