/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.impl.service.internal.system.db.mysql;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.osgi.framework.BundleContext;

public class DBManagerImpl implements org.ugasp.service.internal.system.db.DBManager {
	private Connection connection;
	private boolean isDBConnectivityOK = false;
	
	public DBManagerImpl(BundleContext bc){
		try {
			Driver driver = (Driver) bc.getBundle().loadClass("org.gjt.mm.mysql.Driver").newInstance();
			System.out.println("MySQL Driver loaded: "+driver);
			connection = DriverManager.getConnection("jdbc:mysql://localhost:3306/gaspdb", "gasp", "gasppwd");
			isDBConnectivityOK = true;
			System.out.println("Database connection ok connection: "+connection);
		} 
		catch (ClassNotFoundException e1) {
			System.out.println("Could not load MySQL driver: "+e1.getMessage());
		}
		catch (Exception e){
		    System.out.println("Error loading the database:"+e.getMessage());
		    e.printStackTrace();
		}
	}

	public void stop(){
		try{
			isDBConnectivityOK = false;
			
			if (connection!=null){
				System.out.println("MySQL connection to close: "+connection);
				connection.close();	
			}
			
			System.out.println("Database connection correctly closed");		
		}
		catch (Exception e){
		    System.out.println("Error during database connection closing: "+e.getMessage());
		}
	}
	
	
	/**
	 * Verify if the connection to the GASP DB is ok.
	 * 	
	 * @return boolean
	 */
	public boolean isDBConnectionOK(){ 
	    return isDBConnectivityOK;
	}
	
	
	/**
	 * Generate a DB persistant Actor ID. 
	 * 
	 * @return the actor ID
	 */ 
	public synchronized int generateActorID() {
		int minFreeActorID = 1;
		try {
			Statement statement = (Statement) connection.createStatement();
			
			// search the lowest free actorID in the table actors
			ResultSet result = statement.executeQuery("SELECT actorID FROM actors ORDER BY actorID;");
			while (result.next()){
				int sInt = result.getInt("actorID");
				if (sInt == minFreeActorID) minFreeActorID++;
				if (sInt > minFreeActorID) return Short.parseShort(""+minFreeActorID);
			}
			return minFreeActorID;
		} 
		catch (SQLException e) {e.printStackTrace();}
		return 0;
	}
	
	
	/**
	 * Verify if the login authentification of an Actor is valid 
	 * then return the authentified actor or null.
	 * 
	 * @param aID the actor ID
	 * @param username 
	 * @param password
	 * @return the Actor instance object
	 */
	public Vector getActorIfAuthentificationOK(int aID, String username,String password) {
		try{
			Statement statement = (Statement) connection.createStatement();
			
			// hash the password
			ResultSet result1 = statement.executeQuery("SELECT PASSWORD('"+password+"');");
			result1.first();
			String passwordHashed = result1.getString(1);
			
			// get the actor n-uple matching the aID
			String query = "SELECT * FROM actors WHERE actorID='"+aID+"';";
			ResultSet result2 = statement.executeQuery(query);
			
			// test if the login/pass matching
			if (result2.first()){
				String user = result2.getString("username");
				String pwd = result2.getString("password");
				String pseudo = result2.getString("lastPseudo");
				if (username.equals(user)&&passwordHashed.equals(pwd)){
					
					//Actor actor = new ActorImpl(aID,result2.getInt("appID"),user,pwd,pseudo);
					//return actor;
					Vector v = new Vector();
					v.addElement(result2.getInt("appID"));
					v.addElement(user);
					v.addElement(pwd);
					v.addElement(pseudo);
					
					return v;
				}
			}
		}
		catch (SQLException e) {e.printStackTrace();}
		return null;
	}
	
	/**
	 * Register a new actor for the double (uid,appID) if the user have the appropriate rights.
	 * 
	 * @param uid
	 * @param appID
	 * @param username
	 * @param password
	 * @return the Actor ID
	 */
	public synchronized int registerNewActor(String uid, int appID, String username, String password) {
		int actorID = 0;

		try {
			Statement statement = (Statement) connection.createStatement();
			
			int aIDRegistered = this.userAlreadyRegistered(statement,uid,appID);
			if (aIDRegistered!=0) return aIDRegistered;
			else if (this.isUserAllowed(statement,uid,appID)){

				// generate an actorID
				actorID = this.generateActorID();
				
				// hash the password
				ResultSet result1 = statement.executeQuery("SELECT PASSWORD('"+password+"');");
				result1.first();
				String pwdHashed = result1.getString(1);
				
				// insert the new actor n-uple;
				String query = "INSERT INTO actors VALUES " +
					"("+ actorID +","+ uid +","+ appID +",'"+ username +"','"+ pwdHashed +"','Player');";
				statement.execute(query);
				
				return actorID;
			}
		} 
		catch (SQLException e) {e.printStackTrace(); }
		return 0;
	}

	/**
	 * Determine by search the table rights if the user have already an actorID.
	 * 
	 * @param statement the Statement of DB
	 * @param uid the User ID
	 * @param appID the Application ID
	 * @return boolean
	 */
	private int userAlreadyRegistered(Statement statement, String uid, int appID) {
		try{
			String query = "SELECT * FROM actors WHERE userID='"+ uid +"' AND appID='"+ appID +"'";
			ResultSet result = statement.executeQuery(query);
			if (result.first()) return result.getInt("actorID");
			else return 0;
		}
		catch (SQLException e) {e.printStackTrace();}
		return 0;
	}
	
	
	/**
	 * Determine by search the table rights if the user have the rigths
	 * to play the application appID.
	 * 
	 * @param statement the Statement of DB
	 * @param uid the User ID
	 * @param appID the Application ID
	 * @return boolean
	 */
	private boolean isUserAllowed(Statement statement, String uid, int appID) {
		try{
			String query = "SELECT * FROM rights WHERE userID='"+ uid +"' AND appID='"+ appID +"'";
			ResultSet result = statement.executeQuery(query);
			if (result.first()) return true;
		}
		catch (SQLException e) {e.printStackTrace();}
		return false;
	}

	/**
     * Saves in the DB the last used pseudo.
     * 
     * @param actorID
     * @param pseudoName
     */
    public void saveLastUsedPseudo(int actorID, String pseudoName) {
        try{
            Statement statement = (Statement) connection.createStatement();
			String query = "UPDATE actors SET lastPseudo='"+pseudoName+"' WHERE actorID='"+ actorID +"'";
			statement.execute(query);
		}
        catch (SQLException e) {e.printStackTrace();}
    }
}
