/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.protocol.osc;

import java.util.Date;

import org.ugasp.protocol.osc.OSCPacket;
/**
 * This class provides OSC protocol functionalities to uGASP bundles,
 * e.g. opening in and out ports with distant address.
 * 
 * 
 * @author PELLERIN Romain (romain.pellerin@cnam.fr, romain.pellerin@gmail.com)
 */
public interface OSCBundle extends OSCPacket{
	
	/**
	 * Return the time the bundle will execute.
	 * @return a Date
	 */
	public Date getTimestamp();
	
	/**
	 * Set the time the bundle will execute.
	 * @param timestamp Date
	 */
	public void setTimestamp(Date timestamp);
	
	/**
	 * Add a packet to the list of packets in this bundle.
	 * @param packet OSCMessage or OSCBundle
	 */
	public void addPacket(OSCPacket packet);
	
	/**
	 * Get the packets contained in this bundle.
	 * @return an array of packets
	 */
	public OSCPacket[] getPackets();
}
