/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.protocol.osc;

import java.io.IOException;
import java.net.InetAddress;
import java.util.Date;

/**
 * This class provides OSC protocol functionalities to uGASP bundles,
 * e.g. opening in and out ports with distant address, creating
 * OSC message...
 * 
 * 
 * @author PELLERIN Romain (romain.pellerin@cnam.fr, romain.pellerin@gmail.com)
 */
public interface OSCConnector {
	
	
	/**
	 * Listen to a specific OSC address.
	 * example: oscAdress arg1 arg2 ...
	 * @param packet
	 */
	public void listenTo(String oscAddress, OSCListener listener) throws IOException;
	
	/**
	 * Open an OSC OUT port to the specified client.
	 * @param packet
	 */
	public void openOutlinkTo(InetAddress ipAddress, int port, OSCListener listener) throws IOException;
	
	/**
	 * Two ways OSC connection between a client and the game server logic.
	 * Listen OSC IN port to a specific OSC address, then open the specified OUT port. 
	 * example: oscAdress arg1 arg2 ...
	 * @param packet
	 */
	public void connectTo(InetAddress ipAddress, int port, String oscAddress, OSCListener listener) throws IOException;
	
	/**
	 * Two ways OSC connection between a client and the game server logic.
	 * Listen OSC IN port to a specific IP address, then open the default OUT port (57110). 
	 * example: 127.0.0.1 arg1 arg2 ...
	 * @param packet
	 */
	public void connectTo(InetAddress address, OSCListener listener) throws IOException;
	
	/**
	 * Close OSC connection opened by listener
	 * @param packet
	 */
	public void disconnect(OSCListener listener);
	
	/**
	 * Send an OSC packet
	 * @param packet
	 */
	public void sendPacket(OSCPacket packet, OSCListener listener) throws IOException;
	
	/**
	 * Create an OSC packet
	 * @param packets
	 * @param timestamp
	 */
	public OSCBundle newOSCBundle(OSCPacket[] packets, Date timestamp);
	
	/**
	 * Create an OSC message
	 * @param newAddress
	 * @param newArguments
	 */
	public OSCMessage newOSCMessage(String newAddress, Object[] newArguments);
}
