/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.service.external.player;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.util.Hashtable;

/**
 * When the game programmer wants to use his custom types classes,
 * first he must describe his custom types in the types.xml with the
 * correct syntax. 
 * Then he must run the custom types generator in org.mega.moods package
 * with the command line: MoodsGenerator types.xml packageName
 * The generator generate a file CustomTypes.java that is the encoder/decoder
 * of the game custom types.
 * 
 * 
 * @author PELLERIN Romain (pellerin@cnam.fr) - MEGA Project 
 */
public interface PlayerConnector {
    
	/**
	 * @param sid
	 * @param minActors
	 * @param maxActors
	 * @param actors
	 * @return
	 */
	int create(short sid, int minActors, int maxActors, String[] actors);

	/**
	 * @param asid
	 */
	void end(short asid);

	/**
	 * @param uid
	 * @param appID
	 * @param username
	 * @param password
	 * @return
	 */
	int firstLogin(short uid, short appID, String username,
			String password);

	/**
	 * @param asid
	 * @param data
	 */
	void sendData(short asid, String data);

	/**
	 * @param asid
	 * @return
	 */
	String getEvents(short asid);

	/**
	 * @param asid
	 * @param dis
	 */
	void sendData(short asid, DataInputStream dis);

	/**
	 * @param asid
	 * @param dos
	 */
	void getEvents(short asid, DataOutputStream dos);

	/**
	 * @param sid
	 * @param aiid
	 * @return
	 */
	short joinAI(short sid, short aiid);

	/**
	 * @param sid
	 * @return
	 */
	String lobby(short sid);

	/**
	 * @param sid
	 * @param dos
	 */
	void lobby(short sid, DataOutputStream dos);

	/**
	 * @param aid
	 * @param username
	 * @param password
	 * @return
	 */
	int login(short aid, String username, String password);

	/**
	 * @param asid
	 * @param name
	 * @return
	 */
	String name(short asid, String name);

	/**
	 * @param parseInt
	 */
	void quit(int sid);

	/**
	 * @param asid
	 */
	void quitAI(short asid);

	/**
	 * @param asid
	 */
	boolean start(short asid);

	
	
}
